#!/usr/bin/perl
## ######### PROJECT NAME : ##########
##
## storiq-rescue.pm for storiq-rescue
##
## ######### MODULE DESCRIPTION : ###
##
## Fonctions permettantes de recuperer
## les differentes variables necessaires
## a la modification du fstab au boot.
##
## ###################################
##
## Made by Sardinha Mickael
##
## Started on  Tue May 30 12:36:47 2006 Sardinha Mickael
## Last update Fri Sep 26 16:06:38 2008 Boutonnet Alexandre
##
## ###################################
##

## ###################################
## Declaration de librairies

package Storiq_Rescue;

use strict;
use warnings;
use Term::ANSIColor;
use Tie::File;
use Data::Dumper;

## ###################################
## Methode de recuperation du root en fonction
## du label
# Remonte la parition racine
sub remount_root {
    my $mode = shift;

    eval { qx#mount -o remount,$mode /#; };
    if ($@) {
        return ( 1, $@ );
    }
    return 0;
}

# Gestion des point de montage du fstab
sub do_mountpoint {

    if ( -w "/etc/fstab" ) {

        # Ouverture du fichier fstab
        open( FD, "/etc/fstab" );

        # parcours du fichier
        foreach my $var (<FD>) {

            # On ignore les commentaires
            next if ( $var =~ /^\s*\#/ );

            # On recupere les points de montage
            if ( $var =~ s/^\s*.*\s+(\/\S+).*/$1/ ) {

                chomp($var);

                # On cree les points de montage si ils n'existent pas
                if ( !-d $var ) {
                    mkdir( $var, 0755 ) or die "Error: $!\n";
                }
            }
        }

        # Creation du point de montage /sys si il n'existe pas
        if ( !-d "/sys" ) {
            mkdir( "/sys", 0755 ) or die "Error : $! \n";
        }
    } else {
        return 1;
    }
    return 0;
}

# Creation des devices avec MAKEDEV generic
# CETTE FONCTION EST DEVENUE OBSOLETE
# NOUS UTILISONS UDEV MAINTENANT
sub do_dev {
    if ( !-e "/dev/zero" ) {

        # Si le repertoire /dev n'existe pas on le cree
        if ( !-d "/dev/" ) {
            mkdir( "/dev", 0755 ) or die "Error: $!.\n";
        }

        # Execution du script de creation des devices
        system 'cd /dev; /sbin/MAKEDEV generic';
    }
}

# Gestion du /proc
sub mount_proc {

    # Montage de /proc si /proc/cmdline n'existe pas
    if ( !-f "/proc/cmdline" ) {
        eval { qx#mount -t proc none /proc#; };
        if ($@) {
            return ( 1, $@ );
        }
        return 0;
    } else {
        return 0;
    }
}

# Demonte /proc et supprime son point de montage
sub umount_proc {

    # On demonte /proc

    eval { qx#umount /proc#; };

    if ($@) {
        return ( 1, $@ );
    }
    return 0;
}

# Determination du mode de lancement
sub get_bootmode {
    my $label;
	my $rootdev='/dev/root';
	
	# find the root device
    if ( ! -l $rootdev ) {
		my $cmd = qx#/bin/udevadm info  --device-id-of-file=/ 2>/dev/null# ;
		chomp $cmd;
		my ( $major, $minor ) = split (/:/, $cmd);
		
    	open( my $fh, '<', "/proc/partitions" )
    	  or die "can't parse /proc/partitions: $!";
    	while (<$fh>) {
			my @part = m/\s*(\d+)\s+(\d+)\s+(\d+)\s+([\w|-]+)/;
			if ( $part[0] and $part[0] == $major and $part[1] == $minor) {
				$rootdev = "/dev/$part[3]";
				last;
			}
		}
    }
	
	# find the fs type
    my $fstype;

    open( my $fh, '<', "/proc/mounts" )
      or die "can't parse /proc/mounts: $!";
    while (<$fh>) {
        if (m#/dev/root / (\w+)#g) {

            # on a trouv le root
			$fstype = $1;

            if ( $fstype eq 'reiserfs' ) {
                my @cmd = qx#/sbin/debugreiserfs $rootdev 2>/dev/null#;
                foreach (@cmd) {
                    if (m/LABEL: (.*)/g) {
                        $label = $1;
                        last;
                    }
                }
            } elsif ( $fstype =~ m/ext\d/ ) {
                my @cmd = qx#/sbin/tune2fs -l $rootdev 2>/dev/null#;
                foreach (@cmd) {
                    if (m/Filesystem volume name:\s+(.*)/g) {
                        $label = $1;
                        last;
                    }
                }

            } elsif ( $fstype =~ m/xfs/ ) {
                my @cmd =
                  qx#/usr/sbin/xfs_db -r $rootdev -c 'label' 2>/dev/null#;
                foreach (@cmd) {
                    if (m/label =\s+"(.*)"/g) {
                        $label = $1;
                        last;
                    }
                }
            }
        }
    }

    return $label;

}

# Recuperation du root actuel (l'ancien)
sub get_root_from_fstab {

    # Ouverture en lecture du fstab
    open( FD, "/etc/fstab" ) or die "Error: $!.\n";

    # Parcours du fichier
    while (<FD>) {

        # On ignore les commentaires
        next if ( $_ =~ /^\s*\#/ );

        # Regexp pour recuperer le nouveau root
        if (m/^\s*(\/\w+\/\w+)\s+\/\s+.*/) {
            close(FD);
            return $1;
        }
    }

    # Fermeture du fichier
    close(FD);

    # Sinon c'est quil y a une erreur dans le fichier
    return undef;
}

# Modification du /etc/fstab
sub do_modif {
    my $old = shift;
    my $new = shift;

    # Ouverture en lecture de la copie de fstab
    open( FD_READ, "< /etc/fstab.orig" )
      or die "Error: $!.\n";

    # Ouverture en ecriture du nouveau fstab
    open( FD_WRITE, "> /etc/fstab" )
      or die "Error: $!.\n";

    # On Parcourt le fichier
    while (<FD_READ>) {

        # Si dans le nouveau fstab on match l'ancien root
        # on le remplace par le nouveau
        if ( $_ =~ s/$old/$new/ ) {
            print FD_WRITE "$_";
        }

        # Sinon on remplace rien
        else {
            print FD_WRITE "$_";
        }
    }

    # Fermetures des fichiers
    close(FD_READ);
    close(FD_WRITE);
}

# Recuperation du nouveau root par le lilo.conf
sub get_mountedroot {
	my $rootdev='/dev/root';
	
	# find the root device
    if ( ! -l $rootdev ) {
		my $cmd = qx#/bin/udevadm info  --device-id-of-file=/ 2>/dev/null# ;
		chomp $cmd;
		my ( $major, $minor ) = split (/:/, $cmd);
		
    	open( my $fh, '<', "/proc/partitions" )
    	  or die "can't parse /proc/partitions: $!";
    	while (<$fh>) {
			my @part = m/\s*(\d+)\s+(\d+)\s+(\d+)\s+([\w|-]+)/;
			if ( $part[0] and $part[0] == $major and $part[1] == $minor) {
				$rootdev = "/dev/$part[3]";
				last;
			}
		}
    } else {
		$rootdev=readlink('/dev/root');
		if ($rootdev !~ m#/# ){
			$rootdev = "/dev/$rootdev";
		}
	}

	return $rootdev;
}

# Cette fonction commente la ligne /mnt/rescue
# Cette fonction corrige le bug #287
sub disable_rescue {
    my ($root) = @_;

    # Verification et ouverture du fichier
    my @file;

    unless ( tie( @file, 'Tie::File', "/etc/fstab" ) ) {
        return ( 1, "Unable to open file : /etc/fstab" );
    }

    my $pos = 0;
    while ( $pos < scalar(@file) ) {
        my $line = $file[$pos];

        if ( $line =~ m/^.*$root\s+\/\w+.*$/ ) {
            if ( $line !~ m/^\#.*/ ) {
                splice( @file, $pos++, 0,
                    "# Commented out by Storiq rescue script" );
                $file[$pos] = '# ' . $line;
            }
        }
        $pos++;
    }

    # Close le fichier
    untie(@file);

    return ( 0, 0 );
}

1;
